package ru.ifmo.genetics.dna;

public class AbstractDnaView<D extends LightDna> implements LightDna {
    protected final D dna;
    protected final int begin;
    protected final int end;
    protected final boolean rev;
    protected final boolean compl;

    protected AbstractDnaView(D dna, int begin, int end, boolean rev, boolean compl) {
        this.dna = dna;
        this.begin = begin;
        this.end = end;
        this.rev = rev;
        this.compl = compl;
    }

    protected int getInternalIndex(int index) {
        return (rev ? end - index - 1 : begin + index);
    }

    @Override
    public int length() {
        return end - begin;
    }

    @Override
    public byte nucAt(int index) {
        byte nuc = dna.nucAt(getInternalIndex(index));
        return (byte) (compl ? nuc ^ 3 : nuc);
    }
    
    @Override
    public String toString() {
        return DnaTools.toString(this);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        AbstractDnaView that = (AbstractDnaView) o;

        return DnaTools.equals(this, that);
    }

    @Override
    public int hashCode() {
        return DnaTools.hashCode(this);
    }
}
