package ru.ifmo.genetics.dna;

import java.util.Arrays;

public class DnaQBuilder {
	byte[] data;
	int length;

	public DnaQBuilder() {
		this(16);
	}

	public DnaQBuilder(int capacity) {
		data = new byte[capacity];
	}

	private void ensureCapacity(int minimumCapacity) {
		if (minimumCapacity > data.length) {
			data = Arrays.copyOf(data, Math.max(minimumCapacity, 2 * data.length));
		}
	}
	
	public void unsafeAppend(byte nuc, byte phred) {        
        nuc |= phred << 2;
        data[length++] = nuc;
    }
	
	public void append(byte nuc, byte phred) {
		ensureCapacity(length + 1);
		nuc |= phred << 2;
		data[length++] = nuc;
	}

	public void unsafeAppendUnknown() {
        unsafeAppend((byte) 0, (byte) 0);
    }
    
	
	public void appendUnknown() {
		append((byte) 0, (byte) 0);
	}
	
	public void append(DnaQ dnaQ) {
		ensureCapacity(length + dnaQ.length);
		// TODO faster
		for (int i = 0; i < dnaQ.length; i++) {
			append(dnaQ.nucAt(i), (byte) dnaQ.phredAt(i));
		}
	}
	
	public void append(DnaQ dnaQ, int length) {
		append(dnaQ);
		// TODO faster
		for (int i = dnaQ.length; i < length; i++) {
			appendUnknown();
		}
	}

	public DnaQ build() {
        DnaQ res = new DnaQ(data, 0, length);
        data = null;
		return res;
	}
	
	public byte[] buildToByteArray() {
		return data;
	}

    public int length() {
        return length;
    }
}
