package ru.ifmo.genetics.tools.longReadsAssembler;

import ru.ifmo.genetics.tools.Util;

import java.util.Comparator;
import java.util.TreeMap;

public class AssemblyStat {
	
	private TreeMap<Long, Integer> lengths = new TreeMap<Long, Integer>(new Comparator<Long>() {

		@Override
		public int compare(Long o1, Long o2) {
			return o1 < o2 ? 1 : o1 > o2 ? -1 : 0;
		}
	});
	private long totalLength = 0;
	private long contigsNumber = 0;
	private long[] n = new long[101];
	private boolean outOfDate = false;

	public void add(long length) {
		if (length == 0) {
			System.err.println("What the fuck are you doing here, damned zero-length contig?!!");
			return;
		}
		Util.incrementInt(lengths, length);
		totalLength += length;
		outOfDate = true;
		++contigsNumber;
	}

    public long getContigsNumber() {
        return contigsNumber;
    }
	
	public long getN(int percent) {
		if (percent < 0 || percent > 100) {
			throw new IllegalArgumentException("percent must be an integer in [0, 100] range");
		}
		if (outOfDate) {
			update();
		}
		return n[percent];
	}
	
	public long getMaxLength() {
		return lengths.isEmpty() ? 0 : lengths.firstKey();
	}
	
	public long getMinLength() {
		return lengths.isEmpty() ? 0 : lengths.lastKey();
	}
	
	public long getTotalLength() {
		return totalLength;
	}
	
	public double getMeanLength() {
		return lengths.isEmpty() ? 0 : (((double)totalLength) / contigsNumber);
	}
	
	private void update() {
		int p = 0;
		long curLength = 0;
		for (long length : lengths.keySet()) {
			curLength += length * lengths.get(length);
			int percent = (int)(curLength * 100 / totalLength);
			for (int i = p; i <= percent; ++i) {
				n[i] = length;
			}
			p = percent + 1;
		}
		outOfDate = false;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
        sb.append("total contigs: " + getContigsNumber() + "\n");
		sb.append("total length: " + getTotalLength() + "\n");
		sb.append("maximal length: " + getMaxLength() + "\n");
		sb.append("mean length: " + getMeanLength() + "\n");
		sb.append("minimal length: " + getMinLength() + "\n");
		sb.append("n50: " + getN(50) + "\n");
		sb.append("n90: " + getN(90) + "\n");
//        sb.append("lens: " + lengths);
		
		return sb.toString();
	}

}
