package ru.ifmo.genetics.tools.longReadsAssembler;


import ru.ifmo.genetics.tools.longReadsAssembler.arrays.LargeLongArray;

import java.io.*;
import java.util.Arrays;

// stores nucs in long, small indexes in high bits
public class GluedDnasString {
    //                          indexes =    0,   1,   2,   3,   4,   5,   6,   7
    public final static byte[] charCodes = {'$', 'A', 'C', 'G', 'T'};
//    public final static byte[] charCodes = {'$',  0,   0,   0,  'A', 'C', 'G', 'T'};
    public final static int ALPHABET = charCodes.length;

    public final static byte[] transform = new byte[256];
    static {
        Arrays.fill(transform, (byte) -1);
        for (byte index = 0; index < charCodes.length; index++) {
            transform[charCodes[index]] = index;
        }
        transform[0] = -1;
    }

    public final static byte[] DNAindexes = {transform['A'], transform['C'], transform['G'], transform['T']};
    public final static byte $index = transform['$'];

    public final static int nucWidth = 3;
    public final static int nucsInLong = (Long.SIZE - 1) / nucWidth;

    final static long nucMask = (1 << nucWidth) - 1;
    final static long nucsPackMask = (1L << (nucsInLong * nucWidth)) - 1;

    final static long[] clearMask = new long[nucsInLong];
    static {
        for (int i = 0; i < nucsInLong; i++) {
            clearMask[i] = nucsPackMask ^ (nucMask << (i * nucWidth));
        }
    }

    final static long smallBitInNucsMask;
    static {
        long tmp = 0;
        for (int i = 0; i < nucsInLong; i++) {
            tmp <<= nucWidth;
            tmp |= 1;
        }
        smallBitInNucsMask = tmp;
    }
    /*
    final static long non$maskInPack;
    static {
        long tmp = 0;
        for (int i = 0; i < nucsInLong; ++i) {
            tmp <<= nucWidth;
            tmp += 0x04;
        }
        non$maskInPack = tmp;
    }
    */


    
    LargeLongArray internalArray;
    long length;

    public GluedDnasString(File file) throws IOException {
        this(file.length());
        InputStream br = new BufferedInputStream(new FileInputStream(file));
        byte[] buf = new byte[8192];
        long x = 0;
        while (true) {
            int read = br.read(buf);
            if (read == -1)
                break;
            for (int i = 0; i < read; ++i) {
                set(x++, transform[buf[i]]);
            }
        }
        br.close();
    }

    public long length() {
        return length;
    }
    
    public GluedDnasString(long size) {
        length = size;
        internalArray = new LargeLongArray((size - 1) / nucsInLong + 1);
    }
    
    public void set(long i, int nuc) {
        assert (nuc >>> nucWidth) == 0;
        
        long i1 = i / nucsInLong;
        int i2 = nucsInLong - 1 - (int)(i % nucsInLong);
        
        long x = internalArray.get(i1);
        x &= clearMask[i2];
        x ^= ((long) nuc) << (i2 * nucWidth);
        internalArray.set(i1, x);
    }

    public int get(long i) {
        long i1 = i / nucsInLong;
        int i2 = nucsInLong - 1 - (int)(i % nucsInLong);

        long x = internalArray.get(i1);
        return (int)((x >>> (i2 * nucWidth)) & nucMask);
    }
    
    public long getPackWithLastZeros(long i) {
        long i1 = i / nucsInLong;
        int i2 = (int)(i % nucsInLong);

        long x = internalArray.getWithLastZeros(i1);
        long y = internalArray.getWithLastZeros(i1 + 1);
        long res1 = (x << (i2 * nucWidth)) & nucsPackMask;
        long res2 = (y >>> ((nucsInLong - i2) * nucWidth));
        return res1 | res2;
    }

    static {
        assert $index == 0;
        assert nucWidth == 3;
    }
    public static boolean nucsContain$(long nucs) {
        nucs = nucs | (nucs >>> 1) | (nucs >>> 2);
        return (nucs & smallBitInNucsMask) != smallBitInNucsMask;
    }


    public int getWithLastZeros(long pos) {
        if (pos >= length) {
            return 0;
        }
        return get(pos);
    }

    public String toString(long begin, long end) {
        StringBuilder sb = new StringBuilder();
        for (long i = begin; i < end; i++) {
            sb.append((char)charCodes[get(i)]);
        }
        return sb.toString();
    }

    @Override
    public String toString() {
        return toString(0, length);
    }

}

