package ru.ifmo.genetics.tools.longReadsAssembler.overlaps.overlapper;

import ru.ifmo.genetics.tools.DedicatedWriter;
import ru.ifmo.genetics.tools.longReadsAssembler.GluedDnasString;
import ru.ifmo.genetics.tools.longReadsAssembler.overlaps.Overlaps;
import ru.ifmo.genetics.tools.longReadsAssembler.suffixArray.SuffixArray;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.concurrent.Executor;

public class OverlapTaskContext {

    /**
     * String with all reads, formed as 'read1$read2$...$readN$'.
     */
    public final GluedDnasString text;

    /**
     * Sorted reads.
     */
    public final ArrayList<Integer> readsInBucket;

    /**
     * <code>readBegin[i]</code> = position in <code>text</code> where read number <code>i</code> begins.
     */
    public final long[] readBegin;
    public final int readsNumber;

    public final SuffixArray sa;

    public final Executor executor;

    // for outputting
    public final DedicatedWriter writer;
    public final Overlaps overlaps;

    // overlap algorithm params
    public final int maxNumberOfErrors;
    public final int errorsWindowSize;
    public final int minOverlap;

    public OverlapTaskContext(GluedDnasString text,
                              ArrayList<Integer> readsInBucket,
                              long[] readBegin,
                              int readsNumber,
                              SuffixArray sa,
                              Executor executor,
                              DedicatedWriter writer,
                              Overlaps overlaps,
                              int maxNumberOfErrors,
                              int errorsWindowSize,
                              int minOverlap) {

        this.text = text;
        this.readsInBucket = readsInBucket;
        this.readBegin = readBegin;
        this.readsNumber = readsNumber;
        this.sa = sa;
        this.executor = executor;
        this.writer = writer;
        this.overlaps = overlaps;
        this.maxNumberOfErrors = maxNumberOfErrors;
        this.errorsWindowSize = errorsWindowSize;
        this.minOverlap = minOverlap;
    }
}
