package ru.ifmo.genetics.tools.set;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Arrays;

public class SmallLongsHashSet implements LongsHashSet {
    static final float DEFAULT_LOAD_ASSERTION = 0.90f;

    public static final long FREE = -1;
    
    long[] t;
    int mask;
    int size = 0;
    
    public SmallLongsHashSet(int capacityPowerOf2) {
        t = new long[1 << capacityPowerOf2];
        Arrays.fill(t, FREE);
        
        mask = t.length - 1;
    }
    
    @Override
    public boolean put(long v) {
        int i = get(v);
        if (t[i] == v) {
            return false;
        } else {
            t[i] = v;
            size++;
            if (size > t.length * DEFAULT_LOAD_ASSERTION) {
                throw new AssertionError("High load in small longs hash set");
            }
            return true;
        }
    }
    
    @Override
    public boolean contains(long v) {
        int i = get(v);
        return (t[i] != FREE);  // if v == FREE this condition will return false
    }
    
    int hash(long v) {
        return (int) (v ^ (v >> 3) ^ (v >>> 11) ^ (v >>> 31) ^ (v >>> 57));
    }
    
    /**
     * Returns index for entry with such long value, or free place for it
     */
    int get(long v) {
        int h = hash(v);
        int i = h & mask;
        int c = 0;
        while ((t[i] != v) && (t[i] != FREE)) {
            i = (i + 1) & mask;
            ++c;
        }
        return i;
    }

    @Override
    public long capacity() {
        return t.length;
    }

    @Override
    public void reset() {
        Arrays.fill(t, FREE);
        size = 0;
    }

    @Override
    public long getPosition(long v) {
        int i = get(v);
        return (t[i] != FREE) ? i : -1;
    }
    
    @Override
    public long size() {
        return size;
    }

    @Override
    public long elementAt(long i) {
        return t[(int)i];
    }

    @Override
    public void write(DataOutput out) throws IOException {
        for (int i = 0; i < t.length; ++i) {
            out.writeLong(t[i]);
        }
        out.writeInt(mask);
        out.writeInt(size);

    }

    @Override
    public void readFields(DataInput in) throws IOException {
        for (int i = 0; i < t.length; ++i) {
            t[i] = in.readLong();
        }
        mask = in.readInt();
        size = in.readInt();
    }
}
